import {
  createReactFlowElements,
  dragPill,
  dragPillStarted,
  dragPillStopped,
} from '@graphpolaris/shared/lib/querybuilder/usecases';
import {
  setQuerybuilderNodes,
  useAppDispatch,
  useQuerybuilderNodes,
} from '@graphpolaris/shared/lib/data-access/store';
import ReactFlow, {
  ReactFlowProvider,
  Background,
  Node,
  isNode,
  ReactFlowInstance,
} from 'reactflow';
import styles from './querybuilder.module.scss';
import {
  EntityRFPill,
  RelationRFPill,
  AttributeRFPill,
  ConnectionDragLine,
  ConnectionLine,
} from '@graphpolaris/shared/lib/ui/pills';

import { useMemo, useRef } from 'react';

const nodeTypes = {
  entity: EntityRFPill,
  relation: RelationRFPill,
  attribute: AttributeRFPill,
};
const edgeTypes = {
  connection: ConnectionLine,
};

const onInit = (reactFlowInstance: ReactFlowInstance) => {
  setTimeout(() => reactFlowInstance.fitView(), 0);
};

export const QueryBuilder = (props: any) => {
  const nodes = useQuerybuilderNodes();
  const dispatch = useAppDispatch();
  const isDraggingPill = useRef(false);
  console.log('inputnodes', nodes);

  const elements = useMemo(() => createReactFlowElements(nodes), [nodes]);

  const onNodeDrag = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    // Get the node in the elements list to get the previous location
    const pNode = elements.nodes.find((e) => e.id === node.id);
    if (!(pNode && isNode(pNode))) return;
    // This is then used to calculate the delta position
    const dx = node.position.x - pNode.position.x;
    const dy = node.position.y - pNode.position.y;

    // Check if we started dragging, if so, call the drag started usecase
    if (!isDraggingPill.current) {
      dragPillStarted(node.id, nodes);
      isDraggingPill.current = true;
    }

    // Call the drag usecase
    dragPill(node.id, nodes, dx, dy, node.position);

    // Dispatch the new graphology object, so reactflow will get rerendered
    dispatch(setQuerybuilderNodes(nodes.export()));
  };
  const onNodeDragStop = (
    event: React.MouseEvent<Element, MouseEvent>,
    node: Node<any>
  ) => {
    isDraggingPill.current = false;

    // Call the drag pill stopped usecase
    dragPillStopped(node.id, nodes);

    // Dispatch the new graphology object, so reactflow will get rerendered
    dispatch(setQuerybuilderNodes(nodes.export()));
  };

  console.log(elements);

  return (
    <div
      style={{
        width: '100%',
        height: '100vh',
      }}
    >
      <ReactFlowProvider>
        <ReactFlow
          edges={elements.edges}
          nodes={elements.nodes}
          snapGrid={[10, 10]}
          // snapToGrid
          nodeTypes={nodeTypes}
          edgeTypes={edgeTypes}
          connectionLineComponent={ConnectionDragLine}
          onInit={onInit}
          onNodeDrag={onNodeDrag}
          onNodeDragStop={onNodeDragStop}
          className={styles.reactflow}
        >
          <Background gap={10} size={0.7} />
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default QueryBuilder;
